# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'curveproperties.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CurveProperties(object):
    def setupUi(self, CurveProperties):
        CurveProperties.setObjectName("CurveProperties")
        CurveProperties.resize(342, 280)
        self.verticalLayout = QtWidgets.QVBoxLayout(CurveProperties)
        self.verticalLayout.setObjectName("verticalLayout")
        self.grpName = QtWidgets.QGroupBox(CurveProperties)
        self.grpName.setObjectName("grpName")
        self.formLayout = QtWidgets.QFormLayout(self.grpName)
        self.formLayout.setObjectName("formLayout")
        self.txtName = QtWidgets.QLineEdit(self.grpName)
        self.txtName.setObjectName("txtName")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.txtName)
        self.label_3 = QtWidgets.QLabel(self.grpName)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.btnColor = QtWidgets.QPushButton(self.grpName)
        self.btnColor.setObjectName("btnColor")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.btnColor)
        self.label_4 = QtWidgets.QLabel(self.grpName)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.lblSamplerate = QtWidgets.QLabel(self.grpName)
        self.lblSamplerate.setObjectName("lblSamplerate")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.lblSamplerate)
        self.cmbSymbol = QtWidgets.QComboBox(self.grpName)
        self.cmbSymbol.setObjectName("cmbSymbol")
        self.cmbSymbol.addItem("")
        self.cmbSymbol.addItem("")
        self.cmbSymbol.addItem("")
        self.cmbSymbol.addItem("")
        self.cmbSymbol.addItem("")
        self.cmbSymbol.addItem("")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.cmbSymbol)
        self.label_6 = QtWidgets.QLabel(self.grpName)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.label_2 = QtWidgets.QLabel(self.grpName)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.spnWidth = QtWidgets.QSpinBox(self.grpName)
        self.spnWidth.setObjectName("spnWidth")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.spnWidth)
        self.label_5 = QtWidgets.QLabel(self.grpName)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.label = QtWidgets.QLabel(self.grpName)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.cmbConnect = QtWidgets.QComboBox(self.grpName)
        self.cmbConnect.setObjectName("cmbConnect")
        self.cmbConnect.addItem("")
        self.cmbConnect.addItem("")
        self.cmbConnect.addItem("")
        self.cmbConnect.addItem("")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.cmbConnect)
        self.verticalLayout.addWidget(self.grpName)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.okButton = QtWidgets.QPushButton(CurveProperties)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.okButton.sizePolicy().hasHeightForWidth())
        self.okButton.setSizePolicy(sizePolicy)
        self.okButton.setObjectName("okButton")
        self.horizontalLayout.addWidget(self.okButton)
        self.cancelButton = QtWidgets.QPushButton(CurveProperties)
        self.cancelButton.setObjectName("cancelButton")
        self.horizontalLayout.addWidget(self.cancelButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(CurveProperties)
        QtCore.QMetaObject.connectSlotsByName(CurveProperties)

    def retranslateUi(self, CurveProperties):
        _translate = QtCore.QCoreApplication.translate
        CurveProperties.setWindowTitle(_translate("CurveProperties", "Dialog"))
        self.grpName.setTitle(_translate("CurveProperties", "GroupBox"))
        self.label_3.setText(_translate("CurveProperties", "Color"))
        self.btnColor.setText(_translate("CurveProperties", "Change Color"))
        self.label_4.setText(_translate("CurveProperties", "Sampling rate"))
        self.lblSamplerate.setText(_translate("CurveProperties", "TextLabel"))
        self.cmbSymbol.setItemText(0, _translate("CurveProperties", "None"))
        self.cmbSymbol.setItemText(1, _translate("CurveProperties", "o"))
        self.cmbSymbol.setItemText(2, _translate("CurveProperties", "s"))
        self.cmbSymbol.setItemText(3, _translate("CurveProperties", "t"))
        self.cmbSymbol.setItemText(4, _translate("CurveProperties", "d"))
        self.cmbSymbol.setItemText(5, _translate("CurveProperties", "+"))
        self.label_6.setText(_translate("CurveProperties", "Connect Points"))
        self.label_2.setText(_translate("CurveProperties", "Line width"))
        self.label_5.setText(_translate("CurveProperties", "Symbol"))
        self.label.setText(_translate("CurveProperties", "Name"))
        self.cmbConnect.setItemText(0, _translate("CurveProperties", "None"))
        self.cmbConnect.setItemText(1, _translate("CurveProperties", "All"))
        self.cmbConnect.setItemText(2, _translate("CurveProperties", "Pairs"))
        self.cmbConnect.setItemText(3, _translate("CurveProperties", "Finite"))
        self.okButton.setText(_translate("CurveProperties", "Ok"))
        self.cancelButton.setText(_translate("CurveProperties", "Cancel"))


