from graphysio.plotwidgets.plotwidget import PlotWidget
from graphysio.plotwidgets.puplot import LoopWidget
from graphysio.plotwidgets.poiselector import POISelectorWidget
from graphysio.plotwidgets.spectrogram import SpectrogramWidget
from graphysio.plotwidgets.tsplot import TSWidget
